package chess.client.comm;

import chess.client.gui.ChessClientGUIView;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;

public class ChessClient extends Thread {
    
    private String clientName;
    private String ip;
    private int port;
    
    private Socket socket;
    
    private InputStream is;
    private OutputStream os;
    
    private boolean run;
    
    private byte[] in;
    
    private ChessClientGUIView gui;
    
    public ChessClient(String ip, int port, ChessClientGUIView gui) {
        this.ip = ip;
        this.port = port;
        this.gui = gui;
        try {
            socket = new Socket(ip, port);
            is = socket.getInputStream();
            os = socket.getOutputStream();
            
        } catch (UnknownHostException ex) {
            
        } catch (IOException ex) {
            
        }
        
        gui.updateConnectionState("Connected");
        
    }
    
    @Override
    public void run () {
        int readBytes = 0;
        run = true;
        while (run) {
            try {
                in = new byte[1];
                is.read(in, 0, 1);
                if (readBytes == -1) {
                    run = false;
                    closeConnection();
                    break;
                }
                
                int msg_type = Protocol.readMsgType(in);
                if (msg_type == Protocol.MSG_TYPE_CLIENT_LIST) {
                    in = new byte[Protocol.CLIENT_LIST_SIZE-1];
                    is.read(in, 0, Protocol.CLIENT_LIST_SIZE-1);
                    
                    gui.emptyClientList();
                    String[] client_list = Protocol.readClientList(in);
                    for (String client : client_list) {
                        if (!client.trim().equals(getClientName()))
                            gui.addToClientsList(client.trim());
                    }
                    
                } else {
                    in = new byte[Protocol.PACKET_SIZE-1];
                    is.read(in, 0, Protocol.PACKET_SIZE-1);
                    
                    if (msg_type == Protocol.MSG_TYPE_MOVE) {
                        byte[] move = Protocol.readMoves(in);
                        gui.receivedNewMove(move[0], move[1], move[2], move[3]);
                        
                    } else if (msg_type == Protocol.MSG_TYPE_ERROR) {
                        String err = Protocol.readExtraMessage(in);
                        gui.console(err);
                        
                    } else if (msg_type == Protocol.MSG_TYPE_COLOR) {
                        String color = Protocol.readExtraMessage(in);
                        String oponent = Protocol.readClientTo(in);
                        gui.initBoard(color);
                        gui.selectOpponentAndLockList(oponent);
                        gui.console("Opponent: " + oponent);
                        
                    } else if (msg_type == Protocol.MSG_TYPE_MSG) {
                        String from_client = Protocol.readClientTo(in);
                        String msg = Protocol.readExtraMessage(in);
                        gui.console(from_client + ": " + msg);
                        if (msg.equals("closeconnection")) {
                            gui.unlockList();
                        }
                        
                    } else if (msg_type == Protocol.MSG_TYPE_RESIGN) {
                        gui.console("You win! Opponent reisgned");
                        gui.unlockList();
                        
                    } else if (msg_type == Protocol.MSG_TYPE_DRAW_OFFER) {
                        
                        String extra_msg = Protocol.readExtraMessage(in);
                        
                        if (extra_msg.isEmpty()) {
                            gui.offerDrawQuestion();
                        } else if (extra_msg.equals("yes")) {
                            gui.console("Draw offer accepted");
                            gui.unlockList();
                        }
                    } else if (msg_type == Protocol.MSG_TYPE_MUST_CHAT) {
                        gui.console("Cannot play. Must chat!");
                        gui.enableBoard(false);
                        
                    } else if (msg_type == Protocol.MSG_TYPE_NAME_OK) {
                        HeartBeatClient hb = new HeartBeatClient("localhost", Protocol.HEART_BEAT_PORT, clientName);
                        hb.start();
                        gui.lockUpdateName();
                    }
                }
                
            } catch (IOException ex) {
                closeConnection();
            }
        }
    }
    
    public String getClientName() {
        return clientName;
    }
    
    public void sendMsg(String msg) {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_MSG, getClientName(), "", msg, 1, 2, 3, 4);
        send(data);
    }
    
    public void sendMyName(String name) {
        clientName = name;
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_NAME, getClientName(), "", getClientName(), 1, 2, 3, 4);
        send(data);
    }
    
    public void sendMove(String toClient, int x1, int y1, int x2, int y2) {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_MOVE, getClientName(), toClient, "", x1, y1, x2, y2);
        send(data);
    }
    
    public void sendInvite(String toClient) {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_INVITE, getClientName(), toClient, "", 0, 0, 0, 0);
        send(data);
    }
    
    public void resign() {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_RESIGN, getClientName(), "", "", 0, 0, 0, 0);
        send(data);
    }
    
    public void offerDraw() {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_DRAW_OFFER, getClientName(), "", "", 0, 0, 0, 0);
        send(data);
    }
    
    public void sendDrawReponse(String response) {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_DRAW_OFFER, getClientName(), "", response, 0, 0, 0, 0);
        send(data);
    }
    
    private void send(byte[] data) {
        try {
            os.write(data);
            os.flush();
        } catch (IOException ex) {
            closeConnection();
        }
    }
    
    public void closeConnection() {
        gui.updateConnectionState("Disconnected");
        run = false;
        if (socket != null && os != null) {
            sendMsg("closeconnection");
        }
        try {
            if (is != null) {
                is.close();
                is = null;
            }
            if (os != null) {
                os.close();
                os = null;
            }
            if (socket != null) {
                socket.close();
                socket = null;
            }
        } catch (IOException ex) {
        }
    }
    
    @Override
    public boolean equals(Object o) {
        ChessClient c = (ChessClient)o;
        return c.getClientName().equals(this.getClientName());
    }

}
