package chess.server.comm;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;

public class ChessClient extends Thread {

    private String clientName;
    private Socket socket;
    private InputStream is;
    private OutputStream os;
    private boolean run;
    private byte[] in;
    
    private ClientsManager manager;

    public ChessClient(Socket socket, String clientName, ClientsManager manager) {
        this.socket = socket;
        this.clientName = clientName;
        this.manager = manager;
        try {
            is = socket.getInputStream();
            os = socket.getOutputStream();
        } catch (UnknownHostException ex) {

        } catch (IOException ex) {

        }

    }

    @Override
    public void run() {
        int readBytes = 0;
        run = true;
        while (run) {
            try {
                in = new byte[1];
                readBytes = is.read(in, 0, 1);
                if (readBytes == -1) {
                    run = false;
                    closeConnection();
                    break;
                }

                int msg_type = Protocol.readMsgType(in);
                if (msg_type == Protocol.MSG_TYPE_CLIENT_LIST) {
                    in = new byte[Protocol.CLIENT_LIST_SIZE - 1];
                    is.read(in, 0, Protocol.CLIENT_LIST_SIZE - 1);

                } else {
                    in = new byte[Protocol.PACKET_SIZE - 1];
                    is.read(in, 0, Protocol.PACKET_SIZE - 1);

                    switch (msg_type) {
                        case Protocol.MSG_TYPE_NAME:
                            String name = Protocol.readExtraMessage(in).trim();
                            if (manager.isNameAvailable(name)) {
                                clientName = name;
                                manager.addToTakenNames(name);
                                byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_NAME_OK, "", "", "", 0, 0, 0, 0);
                                send(data);
                            } else {
                                byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_ERROR, "", "", "Name taken. Choose another name", 0, 0, 0, 0);
                                send(data);
                            }
                            manager.sendClientListToClients();
                            break;
                        case Protocol.MSG_TYPE_MSG:
                            String from_client = Protocol.readClientFrom(in);
                            String extra_msg = Protocol.readExtraMessage(in);
                            manager.printToServerConsole(from_client + ": " + extra_msg);

                            if (manager.isClientPaired(from_client)) {
                                //String opponent = pairedPlayers.get(from_client);
                                String opponent = manager.getPairedPlayer(from_client);
                                manager.printToServerConsole("Client " + from_client + " is paired with " + opponent);
                                manager.sendMsgToClient(opponent, from_client, extra_msg);

                            }

                            if (extra_msg.equals("closeconnection")) {
                                manager.closeClientConnection(getClientName());
                            }
                            break;
                        case Protocol.MSG_TYPE_MOVE:
                            from_client = Protocol.readClientFrom(in);
                            manager.printToServerConsole("Received from client " + from_client + "::");
                            String to_client = Protocol.readClientTo(in);

                            if (manager.areClientsPaired(from_client, to_client)) {
                                byte[] move = Protocol.readMoves(in);
                                manager.printToServerConsole(from_client + " move against " + to_client + ": " +
                                        move[0] + "," + move[1] + " " + move[2] + "," + move[3]);
                                manager.sendMoveToClient(from_client, to_client, move);
                            } else {
                                sendErrorMsg(to_client, "Player busy");
                            }
                            break;
                        case Protocol.MSG_TYPE_INVITE:

                            from_client = Protocol.readClientFrom(in);
                            to_client = Protocol.readClientTo(in);

                            if (!manager.isClientPaired(from_client) && !manager.isClientPaired(to_client)) {
                                manager.addToPairedPlayers(from_client, to_client);

                                double ran = Math.random();
                                if (ran < 0.5) {
                                    manager.sendColor(from_client, to_client, "white");
                                    manager.sendColor(to_client, from_client, "black");
                                } else {
                                    manager.sendColor(from_client, to_client, "black");
                                    manager.sendColor(to_client, from_client, "white");
                                }

                            } else {
                                manager.sendMsgToClient(from_client, to_client, to_client + " is busy");
                            }
                            break;
                        case Protocol.MSG_TYPE_RESIGN:
                            if (manager.isClientPaired(getClientName())) {
                                String opponent = manager.getPairedPlayer(getClientName());
                                manager.removePairedPlayers(getClientName(), opponent);
                                manager.sendResignationToClient(opponent);
                            }
                            break;
                        case Protocol.MSG_TYPE_DRAW_OFFER:

                            extra_msg = Protocol.readExtraMessage(in);

                            if (manager.isClientPaired(getClientName())) {
                                String opponent = manager.getPairedPlayer(getClientName());
                                if (extra_msg.isEmpty()) {
                                    manager.sendDrawOfferToClient(opponent);
                                } else if (extra_msg.equals("yes")) {
                                    manager.removePairedPlayers(getClientName(), opponent);
                                    manager.sendDrawOfferResponseToClient(opponent, "yes");
                                    manager.sendDrawOfferResponseToClient(getClientName(), "yes");
                                }
                            }
                            break;

                        }

                }

            } catch (IOException ex) {
                manager.closeClientConnection(getClientName());
            }
        }
    }

    public String getClientName() {
        return clientName;
    }

    public void sendMove(String to_client, byte[] move) {
        System.out.println("Sending move to client " + to_client);
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_MOVE, to_client, getClientName(), "", move);
        send(data);
    }

    public void sendMsg(String from_client, String msg) {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_MSG, getClientName(), from_client, msg, 0, 0, 0, 0);
        send(data);
    }

    public void sendResignation() {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_RESIGN, "", "", "", 0, 0, 0, 0);
        send(data);
    }

    public void sendDrawOffer() {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_DRAW_OFFER, "", "", "", 0, 0, 0, 0);
        send(data);
    }

    public void sendDrawReponse(String response) {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_DRAW_OFFER, getClientName(), "", response, 0, 0, 0, 0);
        send(data);
    }

    public void sendMustChat() {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_MUST_CHAT, getClientName(), "", "", 0, 0, 0, 0);
        send(data);
    }

    private void sendErrorMsg(String to_client, String extra_msg) {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_ERROR, getClientName(), to_client, extra_msg, 0, 0, 0, 0);
        send(data);
    }

    public void send(byte[] data) {
        try {
            os.write(data);
            os.flush();
        } catch (IOException ex) {
            manager.closeClientConnection(getClientName());
        }
    }

    public void closeConnection() {
        System.out.println("Closing connection with client " + getClientName());
        run = false;
        try {
            if (is != null) {
                is.close();
                is = null;
            }
            if (os != null) {
                os.close();
                os = null;
            }
            if (socket != null) {
                socket.close();
                socket = null;
            }
        } catch (IOException ex) {
        }
    }

    @Override
    public boolean equals(Object o) {
        ChessClient c = (ChessClient) o;
        return c.getClientName().equals(this.getClientName());
    }
}
