package chess.server.comm;

import chess.server.gui.*;
import java.net.Socket;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class ClientsManager {

    private static final String HASH = "#_%%&:";
    private int counter;
    private List<ChessClient> clients;
    private ChessServerGUIView gui;
    private Map<String, String> pairedPlayers;
    private Map<String, Integer> moves;
    private Set<String> takenNames;
    private int tempNameCounter;
    private boolean addingNewName;

    public ClientsManager(ChessServerGUIView gui) {
        counter = 1;
        tempNameCounter = 0;
        addingNewName = false;
        clients = new ArrayList<ChessClient>();
        pairedPlayers = new HashMap<String, String>();
        moves = new HashMap<String, Integer>();
        this.gui = gui;
        takenNames = new HashSet<String>();
    }

    public void addNewClient(Socket socket) {
        ChessClient cc = new ChessClient(socket, HASH + tempNameCounter++, this);
        if (!clients.contains(cc)) {
            clients.add(cc);
            clients.get(clients.size() - 1).start();
            gui.console("Client connected and added");
        }
        counter++;
    }

    public synchronized void addToPairedPlayers(String p1, String p2) {
        pairedPlayers.put(p1, p2);
        pairedPlayers.put(p2, p1);
        moves.put(p1, 0);
        moves.put(p2, 0);
    }

    public synchronized void removePairedPlayers(String p1, String p2) {
        pairedPlayers.remove(p1);
        pairedPlayers.remove(p2);
    }

    public synchronized void sendColor(String toclient, String againstClient, String color) {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_COLOR, toclient, againstClient, color, 0, 0, 0, 0);
        for (int i = 0; i < clients.size(); i++) {
            System.out.println(clients.get(i).getClientName() + "," + toclient + "<<");
            if (clients.get(i).getClientName().equals(toclient)) {
                clients.get(i).send(data);
                break;
            }
        }
    }

    public synchronized void sendClientListToClients() {
        String[] cnames = takenNames.toArray(new String[takenNames.size()]);
        byte[] clist = Protocol.createPacketClientList(cnames);
        for (int i = 0; i < clients.size(); i++) {
            clients.get(i).send(clist);
        }

    }

    public synchronized void sendMsgToClient(String clientName, String fromClient, String msg) {
        for (int i = 0; i < clients.size(); i++) {
            if (clients.get(i).getClientName().equals(clientName)) {
                clients.get(i).sendMsg(fromClient, msg);
                break;
            }
        }
    }

    public synchronized void sendResignationToClient(String clientName) {
        for (int i = 0; i < clients.size(); i++) {
            if (clients.get(i).getClientName().equals(clientName)) {
                clients.get(i).sendResignation();
                break;
            }
        }
    }

    public synchronized void sendDrawOfferToClient(String clientName) {
        for (int i = 0; i < clients.size(); i++) {
            if (clients.get(i).getClientName().equals(clientName)) {
                clients.get(i).sendDrawOffer();
                break;
            }
        }
    }

    public void sendDrawOfferResponseToClient(String clientName, String response) {
        for (int i = 0; i < clients.size(); i++) {
            if (clients.get(i).getClientName().equals(clientName)) {
                clients.get(i).sendDrawReponse(response);
                break;
            }
        }
    }

    public void sendMustChatToClient(String clientName) {
        for (int i = 0; i < clients.size(); i++) {
            if (clients.get(i).getClientName().equals(clientName)) {
                clients.get(i).sendMustChat();
                break;
            }
        }
    }

    public void newHeartBeatClient(Socket heartBeatSocket) {
        (new HeartBeatClient(heartBeatSocket, this)).start();
    }

    public boolean areClientsPaired(String c1, String c2) {
        return pairedPlayers.get(c1).equals(c2) ||
                pairedPlayers.get(c2).equals(c1);
    }

    public synchronized boolean isClientPaired(String c) {
        return pairedPlayers.containsKey(c);
    }

    public synchronized void sendMoveToClient(String to_client, String from_client, byte[] move) {
        to_client = to_client.trim();
        from_client = from_client.trim();
        for (int i = 0; i < clients.size(); i++) {
            if (clients.get(i).getClientName().equals(from_client)) {
                clients.get(i).sendMove(to_client, move);
                break;
            }
        }
        moves.put(from_client, (moves.get(from_client) + 1));
        moves.put(to_client, (moves.get(to_client) + 1));

        if (moves.get(from_client) == gui.getNrMovesToChat()) {
            // must chat!
            moves.put(from_client, 0);
            moves.put(to_client, 0);
            sendMustChatToClient(from_client);
            sendMustChatToClient(to_client);
        }
    }

    public void sendToAllClients(String msg) {
        byte[] data = Protocol.createPacket(Protocol.MSG_TYPE_MSG, "", "", msg, 0, 0, 0, 0);
        for (ChessClient c : clients) {
            c.send(data);
        }
    }

    public synchronized boolean isNameAvailable(String name) {
        while (addingNewName == true) {
            try {
                wait();
            } catch (InterruptedException ex) {
            }
        }
        return !takenNames.contains(name);
    }

    public synchronized void addToTakenNames(String name) {
        addingNewName = true;
        takenNames.add(name);
        gui.addToClientsList(name);
        gui.console("Client name updated to: " + name);
        addingNewName = false;
        notifyAll();
    }

    public synchronized void printToServerConsole(String msg) {
        gui.console(msg);
    }

    public synchronized String getPairedPlayer(String client) {
        return pairedPlayers.get(client);
    }
    
    public synchronized void closeClientConnection(String clientName) {
        int i = 0;
        for (i = 0; i < clients.size(); i++) {
            if (clients.get(i).getClientName().equals(clientName)) {
                clients.get(i).closeConnection();
                if (isClientPaired(clientName)) {
                    String pairedPlayer = pairedPlayers.get(clientName);
                    pairedPlayers.remove(clientName);
                    pairedPlayers.remove(pairedPlayer);
                    sendResignationToClient(pairedPlayer);
                }
                gui.removeClientFromList(clientName);
                clients.remove(i);
                takenNames.remove(clientName);
                sendClientListToClients();
                break;
            }
        }
    }
    
}
